# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
"""Declares :class:`OrderEvent`."""
import pydantic

from .event import Event
from .orderstatustype import OrderStatusType


class OrderEvent(Event):
    status: OrderStatusType = pydantic.Field(
        default=...,
        title="Status",
        description="The status of the order resulting from the event."
    )

    class Config:
        title: str = "WooCommerceOrderEvent"