# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import aorta
from aorta.transport import ITransport

from cbra.ext import ioc


class MessagePublisher(aorta.MessagePublisher):

    def __init__(
        self,
        transport: ITransport = ioc.inject('MessageTransport')
    ):
        super().__init__(transport)